
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

/* Automatically generated by glib-mkenums */

#include "dmap-control-share.h"
#include "dmap-control-player.h"
#include "dmap-mdns-publisher.h"
#include "dmap-mdns-browser.h"
#include "dmap-connection.h"
#include "dmap-error.h"
#include "dmap-enums.h"

/* enumerations from "dmap-cc.h" */
GType
dmap_content_code_get_type (void)
{
  static GType etype = 0;
  if (etype == 0) {
    static const GEnumValue values[] = {
      { DMAP_CC_INVALID, "DMAP_CC_INVALID", "cc-invalid" },
      { DMAP_RAW, "DMAP_RAW", "raw" },
      { DMAP_CC_MDCL, "DMAP_CC_MDCL", "cc-mdcl" },
      { DMAP_CC_MEDS, "DMAP_CC_MEDS", "cc-meds" },
      { DMAP_CC_MSTT, "DMAP_CC_MSTT", "cc-mstt" },
      { DMAP_CC_MIID, "DMAP_CC_MIID", "cc-miid" },
      { DMAP_CC_MINM, "DMAP_CC_MINM", "cc-minm" },
      { DMAP_CC_MIKD, "DMAP_CC_MIKD", "cc-mikd" },
      { DMAP_CC_MPER, "DMAP_CC_MPER", "cc-mper" },
      { DMAP_CC_MCON, "DMAP_CC_MCON", "cc-mcon" },
      { DMAP_CC_MCTI, "DMAP_CC_MCTI", "cc-mcti" },
      { DMAP_CC_MPCO, "DMAP_CC_MPCO", "cc-mpco" },
      { DMAP_CC_MSTS, "DMAP_CC_MSTS", "cc-msts" },
      { DMAP_CC_MIMC, "DMAP_CC_MIMC", "cc-mimc" },
      { DMAP_CC_MCTC, "DMAP_CC_MCTC", "cc-mctc" },
      { DMAP_CC_MRCO, "DMAP_CC_MRCO", "cc-mrco" },
      { DMAP_CC_MTCO, "DMAP_CC_MTCO", "cc-mtco" },
      { DMAP_CC_MLCL, "DMAP_CC_MLCL", "cc-mlcl" },
      { DMAP_CC_MLIT, "DMAP_CC_MLIT", "cc-mlit" },
      { DMAP_CC_MBCL, "DMAP_CC_MBCL", "cc-mbcl" },
      { DMAP_CC_MSRV, "DMAP_CC_MSRV", "cc-msrv" },
      { DMAP_CC_MSAU, "DMAP_CC_MSAU", "cc-msau" },
      { DMAP_CC_MSLR, "DMAP_CC_MSLR", "cc-mslr" },
      { DMAP_CC_MPRO, "DMAP_CC_MPRO", "cc-mpro" },
      { DMAP_CC_MSAL, "DMAP_CC_MSAL", "cc-msal" },
      { DMAP_CC_MSUP, "DMAP_CC_MSUP", "cc-msup" },
      { DMAP_CC_MSPI, "DMAP_CC_MSPI", "cc-mspi" },
      { DMAP_CC_MSEX, "DMAP_CC_MSEX", "cc-msex" },
      { DMAP_CC_MSBR, "DMAP_CC_MSBR", "cc-msbr" },
      { DMAP_CC_MSQY, "DMAP_CC_MSQY", "cc-msqy" },
      { DMAP_CC_MSIX, "DMAP_CC_MSIX", "cc-msix" },
      { DMAP_CC_MSRS, "DMAP_CC_MSRS", "cc-msrs" },
      { DMAP_CC_MSTM, "DMAP_CC_MSTM", "cc-mstm" },
      { DMAP_CC_MSDC, "DMAP_CC_MSDC", "cc-msdc" },
      { DMAP_CC_MCCR, "DMAP_CC_MCCR", "cc-mccr" },
      { DMAP_CC_MCNM, "DMAP_CC_MCNM", "cc-mcnm" },
      { DMAP_CC_MCNA, "DMAP_CC_MCNA", "cc-mcna" },
      { DMAP_CC_MCTY, "DMAP_CC_MCTY", "cc-mcty" },
      { DMAP_CC_MLOG, "DMAP_CC_MLOG", "cc-mlog" },
      { DMAP_CC_MLID, "DMAP_CC_MLID", "cc-mlid" },
      { DMAP_CC_MUPD, "DMAP_CC_MUPD", "cc-mupd" },
      { DMAP_CC_MUSR, "DMAP_CC_MUSR", "cc-musr" },
      { DMAP_CC_MUTY, "DMAP_CC_MUTY", "cc-muty" },
      { DMAP_CC_MUDL, "DMAP_CC_MUDL", "cc-mudl" },
      { DMAP_CC_MSMA, "DMAP_CC_MSMA", "cc-msma" },
      { DMAP_CC_FQUESCH, "DMAP_CC_FQUESCH", "cc-fquesch" },
      { DMAP_CC_MDBK, "DMAP_CC_MDBK", "cc-mdbk" },
      { DMAP_CC_APRO, "DMAP_CC_APRO", "cc-apro" },
      { DMAP_CC_AVDB, "DMAP_CC_AVDB", "cc-avdb" },
      { DMAP_CC_ABRO, "DMAP_CC_ABRO", "cc-abro" },
      { DMAP_CC_ABAL, "DMAP_CC_ABAL", "cc-abal" },
      { DMAP_CC_ABAR, "DMAP_CC_ABAR", "cc-abar" },
      { DMAP_CC_ABCP, "DMAP_CC_ABCP", "cc-abcp" },
      { DMAP_CC_ABGN, "DMAP_CC_ABGN", "cc-abgn" },
      { DMAP_CC_ADBS, "DMAP_CC_ADBS", "cc-adbs" },
      { DMAP_CC_ASAL, "DMAP_CC_ASAL", "cc-asal" },
      { DMAP_CC_ASAI, "DMAP_CC_ASAI", "cc-asai" },
      { DMAP_CC_ASAA, "DMAP_CC_ASAA", "cc-asaa" },
      { DMAP_CC_ASAR, "DMAP_CC_ASAR", "cc-asar" },
      { DMAP_CC_ASBT, "DMAP_CC_ASBT", "cc-asbt" },
      { DMAP_CC_ASBR, "DMAP_CC_ASBR", "cc-asbr" },
      { DMAP_CC_ASCM, "DMAP_CC_ASCM", "cc-ascm" },
      { DMAP_CC_ASCO, "DMAP_CC_ASCO", "cc-asco" },
      { DMAP_CC_ASDA, "DMAP_CC_ASDA", "cc-asda" },
      { DMAP_CC_ASDM, "DMAP_CC_ASDM", "cc-asdm" },
      { DMAP_CC_ASDC, "DMAP_CC_ASDC", "cc-asdc" },
      { DMAP_CC_ASDN, "DMAP_CC_ASDN", "cc-asdn" },
      { DMAP_CC_ASDB, "DMAP_CC_ASDB", "cc-asdb" },
      { DMAP_CC_ASEQ, "DMAP_CC_ASEQ", "cc-aseq" },
      { DMAP_CC_ASFM, "DMAP_CC_ASFM", "cc-asfm" },
      { DMAP_CC_ASGN, "DMAP_CC_ASGN", "cc-asgn" },
      { DMAP_CC_ASDT, "DMAP_CC_ASDT", "cc-asdt" },
      { DMAP_CC_ASRV, "DMAP_CC_ASRV", "cc-asrv" },
      { DMAP_CC_ASSR, "DMAP_CC_ASSR", "cc-assr" },
      { DMAP_CC_ASSZ, "DMAP_CC_ASSZ", "cc-assz" },
      { DMAP_CC_ASST, "DMAP_CC_ASST", "cc-asst" },
      { DMAP_CC_ASSP, "DMAP_CC_ASSP", "cc-assp" },
      { DMAP_CC_ASTM, "DMAP_CC_ASTM", "cc-astm" },
      { DMAP_CC_ASTC, "DMAP_CC_ASTC", "cc-astc" },
      { DMAP_CC_ASTN, "DMAP_CC_ASTN", "cc-astn" },
      { DMAP_CC_ASUR, "DMAP_CC_ASUR", "cc-asur" },
      { DMAP_CC_ASYR, "DMAP_CC_ASYR", "cc-asyr" },
      { DMAP_CC_ASDK, "DMAP_CC_ASDK", "cc-asdk" },
      { DMAP_CC_ASUL, "DMAP_CC_ASUL", "cc-asul" },
      { DMAP_CC_ASSU, "DMAP_CC_ASSU", "cc-assu" },
      { DMAP_CC_ASSA, "DMAP_CC_ASSA", "cc-assa" },
      { DMAP_CC_APLY, "DMAP_CC_APLY", "cc-aply" },
      { DMAP_CC_ABPL, "DMAP_CC_ABPL", "cc-abpl" },
      { DMAP_CC_APSO, "DMAP_CC_APSO", "cc-apso" },
      { DMAP_CC_PRSV, "DMAP_CC_PRSV", "cc-prsv" },
      { DMAP_CC_ARIF, "DMAP_CC_ARIF", "cc-arif" },
      { DMAP_CC_MSAS, "DMAP_CC_MSAS", "cc-msas" },
      { DMAP_CC_AGRP, "DMAP_CC_AGRP", "cc-agrp" },
      { DMAP_CC_AGAL, "DMAP_CC_AGAL", "cc-agal" },
      { DMAP_CC_ASCP, "DMAP_CC_ASCP", "cc-ascp" },
      { DMAP_CC_PPRO, "DMAP_CC_PPRO", "cc-ppro" },
      { DMAP_CC_PASP, "DMAP_CC_PASP", "cc-pasp" },
      { DMAP_CC_PFDT, "DMAP_CC_PFDT", "cc-pfdt" },
      { DMAP_CC_PICD, "DMAP_CC_PICD", "cc-picd" },
      { DMAP_CC_PIMF, "DMAP_CC_PIMF", "cc-pimf" },
      { DMAP_CC_PFMT, "DMAP_CC_PFMT", "cc-pfmt" },
      { DMAP_CC_PIFS, "DMAP_CC_PIFS", "cc-pifs" },
      { DMAP_CC_PLSZ, "DMAP_CC_PLSZ", "cc-plsz" },
      { DMAP_CC_PHGT, "DMAP_CC_PHGT", "cc-phgt" },
      { DMAP_CC_PWTH, "DMAP_CC_PWTH", "cc-pwth" },
      { DMAP_CC_PRAT, "DMAP_CC_PRAT", "cc-prat" },
      { DMAP_CC_PCMT, "DMAP_CC_PCMT", "cc-pcmt" },
      { DMAP_CC_PRET, "DMAP_CC_PRET", "cc-pret" },
      { DMAP_CC_AECS, "DMAP_CC_AECS", "cc-aecs" },
      { DMAP_CC_AESV, "DMAP_CC_AESV", "cc-aesv" },
      { DMAP_CC_AEHV, "DMAP_CC_AEHV", "cc-aehv" },
      { DMAP_CC_AESP, "DMAP_CC_AESP", "cc-aesp" },
      { DMAP_CC_AEPP, "DMAP_CC_AEPP", "cc-aepp" },
      { DMAP_CC_AEPS, "DMAP_CC_AEPS", "cc-aeps" },
      { DMAP_CC_AESG, "DMAP_CC_AESG", "cc-aesg" },
      { DMAP_CC_AEMK, "DMAP_CC_AEMK", "cc-aemk" },
      { DMAP_CC_AEMK2, "DMAP_CC_AEMK2", "cc-aemk2" },
      { DMAP_CC_AEFP, "DMAP_CC_AEFP", "cc-aefp" },
      { DMAP_CC_ATED, "DMAP_CC_ATED", "cc-ated" },
      { DMAP_CC_ASGR, "DMAP_CC_ASGR", "cc-asgr" },
      { DMAP_CC_AEMQ, "DMAP_CC_AEMQ", "cc-aemq" },
      { DMAP_CC_AESL, "DMAP_CC_AESL", "cc-aesl" },
      { DMAP_CC_AESR, "DMAP_CC_AESR", "cc-aesr" },
      { DMAP_CC_AETR, "DMAP_CC_AETR", "cc-aetr" },
      { DMAP_CC_MSED, "DMAP_CC_MSED", "cc-msed" },
      { DMAP_CC_CMPA, "DMAP_CC_CMPA", "cc-cmpa" },
      { DMAP_CC_CMNM, "DMAP_CC_CMNM", "cc-cmnm" },
      { DMAP_CC_CMTY, "DMAP_CC_CMTY", "cc-cmty" },
      { DMAP_CC_CMPG, "DMAP_CC_CMPG", "cc-cmpg" },
      { DMAP_CC_CACI, "DMAP_CC_CACI", "cc-caci" },
      { DMAP_CC_CAPS, "DMAP_CC_CAPS", "cc-caps" },
      { DMAP_CC_CASH, "DMAP_CC_CASH", "cc-cash" },
      { DMAP_CC_CARP, "DMAP_CC_CARP", "cc-carp" },
      { DMAP_CC_CAAS, "DMAP_CC_CAAS", "cc-caas" },
      { DMAP_CC_CAAR, "DMAP_CC_CAAR", "cc-caar" },
      { DMAP_CC_CAIA, "DMAP_CC_CAIA", "cc-caia" },
      { DMAP_CC_CANP, "DMAP_CC_CANP", "cc-canp" },
      { DMAP_CC_CANN, "DMAP_CC_CANN", "cc-cann" },
      { DMAP_CC_CANA, "DMAP_CC_CANA", "cc-cana" },
      { DMAP_CC_CANL, "DMAP_CC_CANL", "cc-canl" },
      { DMAP_CC_CANG, "DMAP_CC_CANG", "cc-cang" },
      { DMAP_CC_CANT, "DMAP_CC_CANT", "cc-cant" },
      { DMAP_CC_CASP, "DMAP_CC_CASP", "cc-casp" },
      { DMAP_CC_CASS, "DMAP_CC_CASS", "cc-cass" },
      { DMAP_CC_CAST, "DMAP_CC_CAST", "cc-cast" },
      { DMAP_CC_CASU, "DMAP_CC_CASU", "cc-casu" },
      { DMAP_CC_CASG, "DMAP_CC_CASG", "cc-casg" },
      { DMAP_CC_CACR, "DMAP_CC_CACR", "cc-cacr" },
      { DMAP_CC_CMCP, "DMAP_CC_CMCP", "cc-cmcp" },
      { DMAP_CC_CMGT, "DMAP_CC_CMGT", "cc-cmgt" },
      { DMAP_CC_CMIK, "DMAP_CC_CMIK", "cc-cmik" },
      { DMAP_CC_CMSP, "DMAP_CC_CMSP", "cc-cmsp" },
      { DMAP_CC_CMST, "DMAP_CC_CMST", "cc-cmst" },
      { DMAP_CC_CMSV, "DMAP_CC_CMSV", "cc-cmsv" },
      { DMAP_CC_CMSR, "DMAP_CC_CMSR", "cc-cmsr" },
      { DMAP_CC_CMMK, "DMAP_CC_CMMK", "cc-cmmk" },
      { DMAP_CC_CMVO, "DMAP_CC_CMVO", "cc-cmvo" },
      { DMAP_CC_CMPR, "DMAP_CC_CMPR", "cc-cmpr" },
      { DMAP_CC_CAPR, "DMAP_CC_CAPR", "cc-capr" },
      { DMAP_CC_AEFR, "DMAP_CC_AEFR", "cc-aefr" },
      { DMAP_CC_CAOV, "DMAP_CC_CAOV", "cc-caov" },
      { DMAP_CC_CMRL, "DMAP_CC_CMRL", "cc-cmrl" },
      { DMAP_CC_CAHP, "DMAP_CC_CAHP", "cc-cahp" },
      { DMAP_CC_CAIV, "DMAP_CC_CAIV", "cc-caiv" },
      { DMAP_CC_CAVC, "DMAP_CC_CAVC", "cc-cavc" },
      { 0, NULL, NULL }
    };
    etype = g_enum_register_static ("DmapContentCode", values);
  }
  return etype;
}

/* enumerations from "dmap-connection.h" */
GType
dmap_connection_state_get_type (void)
{
  static GType etype = 0;
  if (etype == 0) {
    static const GEnumValue values[] = {
      { DMAP_GET_INFO, "DMAP_GET_INFO", "get-info" },
      { DMAP_LOGIN, "DMAP_LOGIN", "login" },
      { DMAP_GET_REVISION_NUMBER, "DMAP_GET_REVISION_NUMBER", "get-revision-number" },
      { DMAP_GET_DB_INFO, "DMAP_GET_DB_INFO", "get-db-info" },
      { DMAP_GET_MEDIA, "DMAP_GET_MEDIA", "get-media" },
      { DMAP_GET_PLAYLISTS, "DMAP_GET_PLAYLISTS", "get-playlists" },
      { DMAP_GET_PLAYLIST_ENTRIES, "DMAP_GET_PLAYLIST_ENTRIES", "get-playlist-entries" },
      { DMAP_LOGOUT, "DMAP_LOGOUT", "logout" },
      { DMAP_DONE, "DMAP_DONE", "done" },
      { 0, NULL, NULL }
    };
    etype = g_enum_register_static ("DmapConnectionState", values);
  }
  return etype;
}

/* enumerations from "dmap-control-player.h" */
GType
dmap_control_repeat_state_get_type (void)
{
  static GType etype = 0;
  if (etype == 0) {
    static const GEnumValue values[] = {
      { DMAP_CONTROL_REPEAT_NONE, "DMAP_CONTROL_REPEAT_NONE", "none" },
      { DMAP_CONTROL_REPEAT_SINGLE, "DMAP_CONTROL_REPEAT_SINGLE", "single" },
      { DMAP_CONTROL_REPEAT_ALL, "DMAP_CONTROL_REPEAT_ALL", "all" },
      { 0, NULL, NULL }
    };
    etype = g_enum_register_static ("DmapControlRepeatState", values);
  }
  return etype;
}
GType
dmap_control_play_state_get_type (void)
{
  static GType etype = 0;
  if (etype == 0) {
    static const GEnumValue values[] = {
      { DMAP_CONTROL_PLAY_STOPPED, "DMAP_CONTROL_PLAY_STOPPED", "stopped" },
      { DMAP_CONTROL_PLAY_PAUSED, "DMAP_CONTROL_PLAY_PAUSED", "paused" },
      { DMAP_CONTROL_PLAY_PLAYING, "DMAP_CONTROL_PLAY_PLAYING", "playing" },
      { 0, NULL, NULL }
    };
    etype = g_enum_register_static ("DmapControlPlayState", values);
  }
  return etype;
}

/* enumerations from "dmap-db.h" */
GType
dmap_db_id_get_type (void)
{
  static GType etype = 0;
  if (etype == 0) {
    static const GEnumValue values[] = {
      { DMAP_DB_ID_BAD, "DMAP_DB_ID_BAD", "bad" },
      { 0, NULL, NULL }
    };
    etype = g_enum_register_static ("DmapDbId", values);
  }
  return etype;
}

/* enumerations from "dmap-error.h" */
GType
dmap_error_get_type (void)
{
  static GType etype = 0;
  if (etype == 0) {
    static const GEnumValue values[] = {
      { DMAP_STATUS_OK, "DMAP_STATUS_OK", "ok" },
      { DMAP_STATUS_FAILED, "DMAP_STATUS_FAILED", "failed" },
      { DMAP_STATUS_INVALID_CONTENT_CODE, "DMAP_STATUS_INVALID_CONTENT_CODE", "invalid-content-code" },
      { DMAP_STATUS_INVALID_CONTENT_CODE_SIZE, "DMAP_STATUS_INVALID_CONTENT_CODE_SIZE", "invalid-content-code-size" },
      { DMAP_STATUS_RESPONSE_TOO_SHORT, "DMAP_STATUS_RESPONSE_TOO_SHORT", "response-too-short" },
      { DMAP_STATUS_BAD_FORMAT, "DMAP_STATUS_BAD_FORMAT", "bad-format" },
      { DMAP_STATUS_BAD_BROWSE_CATEGORY, "DMAP_STATUS_BAD_BROWSE_CATEGORY", "bad-browse-category" },
      { DMAP_STATUS_RECORD_MISSING_FIELD, "DMAP_STATUS_RECORD_MISSING_FIELD", "record-missing-field" },
      { DMAP_STATUS_DB_BAD_ID, "DMAP_STATUS_DB_BAD_ID", "db-bad-id" },
      { DMAP_STATUS_OPEN_FAILED, "DMAP_STATUS_OPEN_FAILED", "open-failed" },
      { DMAP_STATUS_CLOSE_FAILED, "DMAP_STATUS_CLOSE_FAILED", "close-failed" },
      { DMAP_STATUS_SEEK_FAILED, "DMAP_STATUS_SEEK_FAILED", "seek-failed" },
      { DMAP_STATUS_NUM_ERRORS, "DMAP_STATUS_NUM_ERRORS", "num-errors" },
      { 0, NULL, NULL }
    };
    etype = g_enum_register_static ("DmapError", values);
  }
  return etype;
}

/* enumerations from "dmap-mdns-browser.h" */
GType
dmap_mdns_browser_error_get_type (void)
{
  static GType etype = 0;
  if (etype == 0) {
    static const GEnumValue values[] = {
      { DMAP_MDNS_BROWSER_ERROR_NOT_RUNNING, "DMAP_MDNS_BROWSER_ERROR_NOT_RUNNING", "not-running" },
      { DMAP_MDNS_BROWSER_ERROR_FAILED, "DMAP_MDNS_BROWSER_ERROR_FAILED", "failed" },
      { 0, NULL, NULL }
    };
    etype = g_enum_register_static ("DmapMdnsBrowserError", values);
  }
  return etype;
}

/* enumerations from "dmap-mdns-publisher.h" */
GType
dmap_mdns_publisher_error_get_type (void)
{
  static GType etype = 0;
  if (etype == 0) {
    static const GEnumValue values[] = {
      { DMAP_MDNS_PUBLISHER_ERROR_NOT_RUNNING, "DMAP_MDNS_PUBLISHER_ERROR_NOT_RUNNING", "not-running" },
      { DMAP_MDNS_PUBLISHER_ERROR_FAILED, "DMAP_MDNS_PUBLISHER_ERROR_FAILED", "failed" },
      { 0, NULL, NULL }
    };
    etype = g_enum_register_static ("DmapMdnsPublisherError", values);
  }
  return etype;
}

/* enumerations from "dmap-mdns-service.h" */
GType
dmap_mdns_service_type_get_type (void)
{
  static GType etype = 0;
  if (etype == 0) {
    static const GEnumValue values[] = {
      { DMAP_MDNS_SERVICE_TYPE_INVALID, "DMAP_MDNS_SERVICE_TYPE_INVALID", "invalid" },
      { DMAP_MDNS_SERVICE_TYPE_DAAP, "DMAP_MDNS_SERVICE_TYPE_DAAP", "daap" },
      { DMAP_MDNS_SERVICE_TYPE_DPAP, "DMAP_MDNS_SERVICE_TYPE_DPAP", "dpap" },
      { DMAP_MDNS_SERVICE_TYPE_DACP, "DMAP_MDNS_SERVICE_TYPE_DACP", "dacp" },
      { DMAP_MDNS_SERVICE_TYPE_RAOP, "DMAP_MDNS_SERVICE_TYPE_RAOP", "raop" },
      { DMAP_MDNS_SERVICE_TYPE_LAST, "DMAP_MDNS_SERVICE_TYPE_LAST", "last" },
      { 0, NULL, NULL }
    };
    etype = g_enum_register_static ("DmapMdnsServiceType", values);
  }
  return etype;
}
GType
dmap_mdns_service_transport_protocol_get_type (void)
{
  static GType etype = 0;
  if (etype == 0) {
    static const GEnumValue values[] = {
      { DMAP_MDNS_SERVICE_TRANSPORT_PROTOCOL_TCP, "DMAP_MDNS_SERVICE_TRANSPORT_PROTOCOL_TCP", "tcp" },
      { DMAP_MDNS_SERVICE_TRANSPORT_PROTOCOL_UDP, "DMAP_MDNS_SERVICE_TRANSPORT_PROTOCOL_UDP", "udp" },
      { DMAP_MDNS_SERVICE_TRANSPORT_PROTOCOL_LAST, "DMAP_MDNS_SERVICE_TRANSPORT_PROTOCOL_LAST", "last" },
      { 0, NULL, NULL }
    };
    etype = g_enum_register_static ("DmapMdnsServiceTransportProtocol", values);
  }
  return etype;
}

/* enumerations from "dmap-record.h" */
GType
dmap_media_kind_get_type (void)
{
  static GType etype = 0;
  if (etype == 0) {
    static const GEnumValue values[] = {
      { DMAP_MEDIA_KIND_MUSIC, "DMAP_MEDIA_KIND_MUSIC", "music" },
      { DMAP_MEDIA_KIND_MOVIE, "DMAP_MEDIA_KIND_MOVIE", "movie" },
      { DMAP_MEDIA_KIND_PODCAST, "DMAP_MEDIA_KIND_PODCAST", "podcast" },
      { DMAP_MEDIA_KIND_TV_SHOW, "DMAP_MEDIA_KIND_TV_SHOW", "tv-show" },
      { 0, NULL, NULL }
    };
    etype = g_enum_register_static ("DmapMediaKind", values);
  }
  return etype;
}

/* enumerations from "dmap-share.h" */
GType
dmap_share_auth_method_get_type (void)
{
  static GType etype = 0;
  if (etype == 0) {
    static const GEnumValue values[] = {
      { DMAP_SHARE_AUTH_METHOD_NONE, "DMAP_SHARE_AUTH_METHOD_NONE", "none" },
      { DMAP_SHARE_AUTH_METHOD_NAME_AND_PASSWORD, "DMAP_SHARE_AUTH_METHOD_NAME_AND_PASSWORD", "name-and-password" },
      { DMAP_SHARE_AUTH_METHOD_PASSWORD, "DMAP_SHARE_AUTH_METHOD_PASSWORD", "password" },
      { 0, NULL, NULL }
    };
    etype = g_enum_register_static ("DmapShareAuthMethod", values);
  }
  return etype;
}

/* Generated data ends here */

